@@ -1,11 +1,9 @@
 package Mojar::Mysql;
-use strict;
-use version;
-use 5.010001;
+use Mojo::Base -strict;
 
-our $VERSION = 2.034;
+our $VERSION = 2.042;
 
-1
+1;
 __END__
 
 =head1 NAME
@@ -15,5 +13,3 @@ Mojar::Mysql - Suite of MySQL integration tools.
 =head1 DESCRIPTION
 
 This initial release contains the database connector L<Mojar::Mysql::Connector>.
-
-=cut
